/*
    Title:    1,1 Zoll Pictiva mit ATMEGA32
	Version:  1.00
    Author:   Jens Dietrich
    Start:    06/01/2007
    Date:     07/01/2007
    Purpose:  3,3 Volt
    Software: AVR-GCC
    Hardware: ATMEGA32 + 3,3 Volt + 1 zoll SSD1303
    Note:     http://www.icplan.de
	File:     pictiva.c
*/

#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "pictiva.h"

int main(void)
{
	PORTA = 0x00;			    									/* port a ausgaenge auf low */
	DDRA =0xff;  				    								/* port a output */
	
	DDRC = 0xfc;													/* bit 0,1 I2C rest ausgaenge */

	PORTD = 0b00000010;		    									/* port c bit txd mit pulup */
	DDRD =  0b11111100; 		                              		/* bit 2-7 output ; bit 0,1 input rs232 */
	PORTD = 0x00;
	DDRD =  0b11111110; 		                              		/* bit 1-7 output ; bit 0 input rs232 */

	TWBR = 0x80;													/* i2c bitrate etwa 40 khz */
	UBRRL = 103;													/* 51=8 103=16 mhz bei 9600 baud */
	UCSRB = 0x98;													/* rx, tx + inter bei rx einschalten */
 	
	TCCR1B = 0x03;	    											/* CK/8 = 0x02 */
	TIMSK = 0x04;		    										/* timer 1 overflow enable */

    PORTA &= ~(0x08);
    delay(10);
    PORTA |= (0x08);												/* reset display */
    delay(200);

// pictiva 1303 display laut herstellerangaben initialisieren

	send_data(0x00); 												/* Set Lower Column Address */
	send_data(0x10); 												/* Set Higher Column Address */

	send_data(0x40); 												/* Set Display Start Line */

	send_data(0x81); 												/* Set Contrast Control */
	send_data(0x30); 												/* 0 ~ 127 */

	send_data(0xA0); 												/* [A0]:column address 0 is */
	send_data(0xC8); 												/* oben / unten */

	send_data(0xA4); 												/* A4=ON */

	send_data(0xA6);  												/* Normal Display*/

	send_data(0xA8); 												/* Set Multiplex Ratio */
	send_data(0x3f);

	send_data(0xAD); 												/* Set DC-DC */
	send_data(0x8A); 												/* 8B=ON, 8A=Off */

	send_data(0xAF);  												/* AF=ON , AE=OFF*/

	send_data(0xD3); 												/* Set Display Offset */
	send_data(0x00); 												/* No offset */

	send_data(0xD5); 												/* Set Clock Divide */
	send_data(0x61); 												/* Set to 80Hz */

	send_data(0xD8); 												/* Set Area Color On or Off */
	send_data(0x00); 												/* Mono Mode */

	send_data(0xDA); 												/* Set Pins Hardware */
	send_data(0x12);

	send_data(0xDB); 												/* Set VCOMH */
	send_data(0x00);

	send_data(0xD9); 												/* Set VP */
	send_data(0x22); 												/* P1=2 , P2=2 */
    
	oled_clear();

	zeichen_big2('A',0,0);
	zeichen_big2('B',1,0);

	zeichen_big('A',0,2);
	zeichen_big('B',1,2);
	zeichen_big('C',2,2);
	zeichen_big('D',3,2);
	zeichen_big('E',0,3);
	zeichen_big('F',1,3);
	zeichen_big('G',2,3);
	zeichen_big('H',3,3);

	unsigned char w=0x40;			
    unsigned char u=0,v=0;
    for(v=0;v<8;v++)
	{
	  for(u=8;u<16;u++)
      {
		zeichen (w++,u,v);
		if(w==0x80)w=0x20;
		delay(200);													// zeichen langsam anzeigen
	  }
    }

    for (;;)
	{
	};
}

void oled_clear(void)
{
  unsigned char a, b;
  for(a=0;a<8;a++)
	for(b=0;b<16;b++)zeichen(' ',b,a);
}

// zeichenausgabe vierfache hhe an bestimmte displayposition - display hat jetzt nur 4 spalten/zeichen und 2 zeilen
// z=zeichen 0x20-0x7f ; spalte=0-3 ; zeile=0-1

void zeichen_big2 (unsigned char z, unsigned char spalte, unsigned char zeile)
{
  unsigned char a, b, c, d, e, f;
  unsigned int stelle;

  if((z<0x20)||(z>0x7f))z=0x20;
  stelle = 6*(z-0x20);
  
  a = 24*(spalte&0x03);
  e = (a&0x0f);
  send_data(e);														// low spalte setzen
  f = ((((a&0xf0)>>4)+0x10));
  send_data(f);						  								// high spalte setzen
  d = 4*(zeile&0x01);

  for(c=0;c<6;c++)
  {
    a = pgm_read_byte(&z_code[stelle++]); 
    send_data(0xb0 + d);											// erste zeile setzen
    b = 0;
    if(a&0x01)b|=0x0f;												// die vier doppelten punkte setzen
    if(a&0x02)b|=0xf0;
    send_char(b);
    send_char(b);
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl
    send_data(e);													// low spalte setzen
    send_data(f);													// high spalte setzen
    send_data(0xb1 + d);											// zweite zeile setzen
	b = 0;
    if(a&0x04)b|=0x0f;
    if(a&0x08)b|=0xf0;
    send_char(b);
    send_char(b);
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl
    send_data(e);													// low spalte setzen
    send_data(f);													// high spalte setzen
    send_data(0xb2 + d);											// dritte zeile setzen
	b = 0;
    if(a&0x10)b|=0x0f;
    if(a&0x20)b|=0xf0;
    send_char(b);
    send_char(b);
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl
    send_data(e);													// low spalte setzen
    send_data(f);													// high spalte setzen
    send_data(0xb3 + d);											// vierte zeile setzen
	b = 0;
    if(a&0x40)b|=0x0f;
    if(a&0x80)b|=0xf0;
    send_char(b);
    send_char(b);
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl

	e = e + 4;														// spaltenadresse 4 fach erhhen
	if(e>0x0f)
	{
	  e = e & 0x0f;
	  f = f + 1;
	}
  }
}

// zeichenausgabe doppelte hhe an bestimmte displayposition - display hat jetzt nur 8 spalten/zeichen und 4 zeilen
// z=zeichen 0x20-0x7f ; spalte=0-7 ; zeile=0-3

void zeichen_big (unsigned char z, unsigned char spalte, unsigned char zeile)
{
  unsigned char a, b, c, d, e, f;
  unsigned int stelle;

  if((z<0x20)||(z>0x7f))z=0x20;
  stelle = 6*(z-0x20);
  
  a = 12*(spalte&0x07);
  e = (a&0x0f);
  send_data(e);														// low spalte setzen
  f = ((((a&0xf0)>>4)+0x10));
  send_data(f);						  								// high spalte setzen
  d = 2*(zeile&0x03);

  for(c=0;c<6;c++)
  {
    a = pgm_read_byte(&z_code[stelle++]); 
    send_data(0xb0 + d);											// obere zeile setzen
    b = 0;
    if(a&0x01)b|=0x03;												// die vier doppelten punkte setzen
    if(a&0x02)b|=0x0c;
    if(a&0x04)b|=0x30;
    if(a&0x08)b|=0xc0;
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl
    send_data(e);													// low spalte setzen
    send_data(f);													// high spalte setzen
    send_data(0xb1 + d);											// untere zeile setzen
    b = 0;
    if(a&0x10)b|=0x03;												// die vier doppelten punkte setzen
    if(a&0x20)b|=0x0c;
    if(a&0x40)b|=0x30;
    if(a&0x80)b|=0xc0;
    send_char(b);
    send_char(b);
    send_data(0xe3);												// nop befehl
	
	e = e + 2;														// spaltenadresse 2 fach erhhen
	if(e>0x0f)
	{
	  e = e & 0x0f;
	  f = f + 1;
	}
  }
}

// zeichenausgabe an bestimmte displayposition - display hat 16 spalten/zeichen und 8 zeilen
// z=zeichen 0x20-0x7f ; spalte=0-15 ; zeile=0-7

void zeichen (unsigned char z, unsigned char spalte, unsigned char zeile)
{
  unsigned char a;
  unsigned int stelle;

  if((z<0x20)||(z>0x7f))z=0x20;
  stelle = 6*(z-0x20);
  
  a = 6*(spalte&0x0f);
  send_data(a&0x0f);												// low spalte setzen
  send_data((((a&0xf0)>>4)+0x10));  								// high spalte setzen
  a = (zeile&0x07);
  send_data(0xb0 + a);												// zeile setzen
  
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  a = pgm_read_byte(&z_code[stelle++]); 
  send_char(a);
  send_data(0xe3);													// nop befehl
}

void send_char(unsigned char zeichen)
{
  PORTA |= (0x04);		
  senden_spi(zeichen);
}

void send_data(unsigned char zeichen)
{
  PORTA &= ~(0x04);	
  senden_spi(zeichen);
  PORTA |= (0x04);		
}

void senden_spi(unsigned char zeichen)
{
  unsigned char a=0, b=0b10000000;
  PORTA &= ~(0x10);	
  PORTA |= (0x02);		

  for(a=0;a<8;a++)
  { 
    if(zeichen&b)PORTA |= (0x01);
    else PORTA &= ~(0x01);
	PORTA &= ~(0x02);												// clock low
    PORTA |= (0x02);												// clock high 
    b=b>>1;
  }
  PORTA |= (0x10);													// cs high
}

void delay(unsigned char d)										// zeitschleife
{
  unsigned char i, j;
  for (i=0; i<d; i++)
	for (j=0; j<255; j++);
}



